@@ -1,19 +1,23 @@
 # coding: utf-8
 # create by tongshiwei on 2019-9-1
 
-__all__ = ["get_net", "get_bp_loss"]
+__all__ = ["get_net", "get_loss"]
 
 # todo: define your network symbol and back propagation loss function
 
 from mxnet import gluon
 
-
-def get_net(**kwargs):
-    return NetName(**kwargs)
+from longling.ML.MxnetHelper.toolkit import loss_dict2tmt_mx_loss
 
 
-def get_bp_loss(**kwargs):
-    return {"L2Loss": gluon.loss.L2Loss(**kwargs)}
+def get_net(*args, **kwargs):
+    return NetName(*args, **kwargs)
+
+
+def get_loss(**kwargs):
+    return loss_dict2tmt_mx_loss({
+        "L2Loss": gluon.loss.L2Loss(**kwargs)
+    })
 
 
 class NetName(gluon.HybridBlock):
