@@ -4,22 +4,24 @@
 """
 自定义的配置文件及对应的解析工具包。目的是为了更方便、快速地进行文件参数配置与解析。
 """
-
 import argparse
 import inspect
 import json
 import logging
 import os
 import re
-from collections import Iterable
 import sys
+from collections import Iterable
+from copy import deepcopy
 
+import toml
+import yaml
 from longling import wf_open
 from longling.lib.path import path_append
 
 __all__ = [
     "CLASS_EXCLUDE_NAMES", "get_class_var",
-    "get_parsable_var", "load_configuration_json",
+    "get_parsable_var", "load_configuration",
     "var2exp", "path_append",
     "Configuration", "ConfigurationParser",
     "Formatter"
@@ -30,7 +32,7 @@ CLASS_EXCLUDE_NAMES = set(vars(object).keys()) | {
 }
 
 
-def get_class_var(class_obj, exclude_names: (set, None) = None) -> dict:
+def get_class_var(class_obj, exclude_names: (set, None) = None, get_vars=True) -> dict:
     """
     获取某个类的所有属性的变量名及其值
 
@@ -58,6 +60,7 @@ def get_class_var(class_obj, exclude_names: (set, None) = None) -> dict:
         类或类实例。需要注意两者的区别。
     exclude_names:
         需要排除在外的变量名。也可以通过在类定义 excluded_names 方法来指定要排除的变量名。
+    get_vars
 
     Returns
     -------
@@ -71,10 +74,18 @@ def get_class_var(class_obj, exclude_names: (set, None) = None) -> dict:
         class_obj, "excluded_names"
     ) else class_obj.excluded_names() | default_excluded_names
 
-    variables = {
-        k: v for k, v in vars(class_obj).items() if
-        not inspect.isroutine(v) and k not in excluded_names
-    }
+    if get_vars:
+        variables = {
+            k: v for k, v in vars(class_obj).items() if
+            not inspect.isroutine(v) and k not in excluded_names
+        }
+    else:
+        variables = {}
+        for k in dir(class_obj):
+            if hasattr(class_obj, k):
+                v = getattr(class_obj, k)
+                if not inspect.isroutine(v) and k not in excluded_names:
+                    variables[k] = v
     return variables
 
 
@@ -101,15 +112,33 @@ def parse_params(params: dict, parse_functions: (dict, None) = None) -> dict:
     return params
 
 
-def get_parsable_var(class_obj, parse_exclude: set = None, dump_parse_functions=None):
+def get_parsable_var(class_obj, parse_exclude: set = None, dump_parse_functions=None, get_vars=True):
     """获取所有可以被解析的参数及其值，可以使用dump_parse_functions来对不可dump的值进行转换"""
-    params = get_class_var(class_obj, exclude_names=parse_exclude)
+    params = get_class_var(class_obj, exclude_names=parse_exclude, get_vars=get_vars)
     return parse_params(params, dump_parse_functions)
 
 
-def load_configuration_json(fp, load_parse_function=None):
-    """装载配置文件"""
-    params = json.load(fp)
+def load_configuration(fp, file_format="json", load_parse_function=None):
+    """
+    装载配置文件
+
+    Updated in version 1.3.16
+
+    Parameters
+    ----------
+    fp
+    file_format
+    load_parse_function
+
+    """
+    if file_format == "json":
+        params = json.load(fp)
+    elif file_format == "toml":
+        params = toml.load(fp)
+    elif file_format == "yaml":
+        params = yaml.load(fp)
+    else:
+        raise TypeError("Unsupported file format: %s, only `json`, `toml` and `yaml` are supported" % file_format)
     return parse_params(params, load_parse_function)
 
 
@@ -135,7 +164,7 @@ def var2exp(var_str, env_wrap=lambda x: x):
 
     env_vars = pattern.findall(var_str)
 
-    exp = """str("%s").format(%s)""" % (
+    exp = """str(r"%s").format(%s)""" % (
         re.sub(r"\$\w+", "{}", var_str), ",".join(
             [env_wrap(env_var) for env_var in env_vars]
         )
@@ -203,7 +232,8 @@ class Configuration(object):
 
     @classmethod
     def vars(cls):
-        return get_class_var(cls)
+        # todo: check why when deepcopy is missed, two instance will have same id for class_var
+        return deepcopy(get_class_var(cls))
 
     @property
     def class_var(self):
@@ -241,29 +271,69 @@ class Configuration(object):
             dump_parse_functions=None,
         )
 
-    @staticmethod
-    def load_cfg(params_json, **kwargs):
+    @classmethod
+    def load_parse_function(cls):
+        return None
+
+    @classmethod
+    def default_file_format(cls):
+        return "json"
+
+    @classmethod
+    def load_cfg(cls, cfg_path, file_format=None, **kwargs):
         """从配置文件中装载配置参数"""
-        with open(params_json) as f:
-            params = load_configuration_json(
-                f, load_parse_function=None
+        file_format = file_format if file_format is not None else cls.default_file_format()
+
+        with open(cfg_path) as f:
+            params = load_configuration(
+                f, file_format=file_format, load_parse_function=cls.load_parse_function()
             )
         params.update(kwargs)
         return params
 
     @classmethod
-    def load(cls, cfg_path, **kwargs):
-        """从配置文件中装载配置类"""
-        return cls(**cls.load_cfg(cfg_path, **kwargs))
+    def load(cls, cfg_path, file_format=None, **kwargs):
+        """
+        从配置文件中装载配置类
 
-    def dump(self, cfg_json: str, override=False):
-        """将配置参数写入文件"""
-        if os.path.isfile(cfg_json) and not override:
-            self.logger.warning("file %s existed, dump aborted" % cfg_json)
+        Updated in version 1.3.16
+        """
+        file_format = file_format if file_format is not None else cls.default_file_format()
+        return cls(**cls.load_cfg(cfg_path, file_format=file_format, **kwargs))
+
+    def dump(self, cfg_path: str, override=True, file_format=None):
+        """
+        将配置参数写入文件
+
+        Updated in version 1.3.16
+
+        Parameters
+        ----------
+        cfg_path: str
+        override: bool
+        file_format: str
+        """
+        if os.path.isfile(cfg_path) and not override:
+            self.logger.warning(
+                "file %s existed, dump aborted" % os.path.abspath(cfg_path)
+            )
             return
-        self.logger.info("writing configuration parameters to %s" % cfg_json)
-        with wf_open(cfg_json) as wf:
-            json.dump(self.parsable_var, wf, indent=4)
+        self.logger.info(
+            "writing configuration parameters to %s" % os.path.abspath(cfg_path)
+        )
+        file_format = file_format if file_format is not None else self.default_file_format()
+
+        with wf_open(cfg_path) as wf:
+            if file_format == "json":
+                json.dump(self.parsable_var, wf, indent=2)
+            elif file_format == "toml":
+                toml.dump(self.parsable_var, wf)
+            elif file_format == "yaml":
+                yaml.dump(self.parsable_var, wf)
+            else:
+                raise TypeError(
+                    "Unsupported file format: %s, only `json`, `toml` and `yaml` are supported" % file_format
+                )
 
     def __str__(self):
         string = []
@@ -280,8 +350,12 @@ class Configuration(object):
     def items(self):
         return {k: v for k, v in self.parsable_var.items()}
 
-    @staticmethod
-    def excluded_names():
+    @classmethod
+    def run_time_variables(cls):
+        return {'logger'}
+
+    @classmethod
+    def excluded_names(cls):
         """
         获取非参变量集
 
@@ -292,7 +366,30 @@ class Configuration(object):
         """
         return CLASS_EXCLUDE_NAMES | {
             'class_var', 'parsable_var', 'items', 'load', 'dump'
-        } | {'logger'}
+        } | cls.run_time_variables()
+
+
+class InheritableConfiguration(Configuration):
+    @classmethod
+    def vars(cls):
+        return get_class_var(cls, get_vars=False)
+
+    @property
+    def parsable_var(self):
+        """
+        获取可以进行命令行设定的参数
+
+        Returns
+        -------
+        store_vars: dict
+            可以进行命令行设定的参数
+        """
+        return get_parsable_var(
+            self,
+            parse_exclude=None,
+            dump_parse_functions=None,
+            get_vars=False
+        )
 
 
 def value_parse(value):
@@ -709,16 +806,3 @@ class Formatter(object):
     def format(*format_string, formatter=None):
         formatter = Formatter(formatter)
         return formatter(*format_string)
-
-
-if __name__ == '__main__':
-    class TestCls(Configuration):
-        a = 1
-
-        @classmethod
-        def cls_m(cls, new_a):
-            cls.a = new_a
-
-    parser = ConfigurationParser(TestCls)
-    parser.add_command(TestCls.cls_m)
-    print(parser("cls_m 1"))
